'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.addPrefixedProperties = addPrefixedProperties;
exports.getPrefixedValue = getPrefixedValue;
exports.getPrefixedProperty = getPrefixedProperty;
exports.caplitalizeString = caplitalizeString;
exports.isPrefixProperty = isPrefixProperty;
exports.resolveHack = resolveHack;
exports.generateRequiredProperties = generateRequiredProperties;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _browserinfo = require('./browserinfo');

var _browserinfo2 = _interopRequireDefault(_browserinfo);

var _caniuseData = require('../caniuseData');

var _caniuseData2 = _interopRequireDefault(_caniuseData);

var _hacks = require('./hacks');

var _hacks2 = _interopRequireDefault(_hacks);

var browserinfo = undefined;
var requiredHacks = [];
var requiredProperties = [];
var defaultUserAgent = typeof navigator !== 'undefined' ? navigator.userAgent : undefined;
var lastUserAgent = undefined;
var generated = false;

/**
 * Transforms camel case to dash case (param case)
 * @param {string} str - str that gets transformed
 * Thanks to @ianobermiller for this short method
 * https://github.com/rofrischmann/inline-style-prefixer/issues/9
 */
var CAMEL_CASE_REGEXP = /([a-z]|^)([A-Z])/g;
var camelToDashCase = function camelToDashCase(str) {
	return str.replace(CAMEL_CASE_REGEXP, function (match, p1, p2) {
		return p1 + '-' + p2.toLowerCase();
	});
};

/**
 * Processes an object of styles using userAgent specific
 * @param {Object} styles - Styles object that gets prefixed
 * @param {Boolean} hacks - If hacks should be used to resolve browser differences
 */

exports['default'] = function (styles) {
	var userAgent = arguments.length <= 1 || arguments[1] === undefined ? defaultUserAgent : arguments[1];

	if (lastUserAgent !== userAgent) {
		generated = false;
		generateRequiredProperties(userAgent);
	}

	//only add prefixes if needed
	if (requiredProperties.length > 0) {
		addPrefixedProperties(styles);
	}
	return styles;
};

/**
 * Adds prefixed properties to a style object
 * @param {Object} styles - Style object that gets prefixed properties added
 */

function addPrefixedProperties(styles) {
	Object.keys(styles).forEach(function (property) {
		var value = styles[property];
		if (value instanceof Object) {
			//recursively loop through nested style objects
			addPrefixedProperties(value);
		} else {
			//add prefixes if needed
			if (isPrefixProperty(property)) {
				styles[getPrefixedProperty(property)] = value;
			}
			//resolve hacks
			requiredHacks.forEach(function (hack) {
				resolveHack(hack, styles, property, value);
			});
		}
	});

	return styles;
}

/**
 * Returns a prefixed value
 * Optionaly uses an alternative value
 * @param {string} property - CSS property that gets prefixed
 * @param {any} value - old value that gets prefixed
 * @param {any} alternative - alternative value used for prefixing 
 */

function getPrefixedValue(property, value, alternative) {
	if (alternative) {
		return value.replace(value, browserinfo.prefix.CSS + alternative, 'g') + ';' + camelToDashCase(property) + ':' + value;
	} else {
		return browserinfo.prefix.CSS + value + ';' + camelToDashCase(property) + ':' + value;
	}
}

/**
 * Returns a prefixed style property
 * @param {String} property - a style property in camelCase
 * @param {String} prefix - evaluated vendor prefix that will be added
 */

function getPrefixedProperty(property) {
	return browserinfo.prefix.inline + caplitalizeString(property);
}

/**
 * Capitalizes first letter of a string
 * @param {String} str - str to caplitalize
 */

function caplitalizeString(str) {
	return str.charAt(0).toUpperCase() + str.slice(1);
}

/**
 * Checks if a property needs to be prefixed
 * @param {String} property - a style property
 */

function isPrefixProperty(property) {
	return requiredProperties.indexOf(property) > -1;
}

/**
 * Resolves browser issues using some hacks
 * @param {Object} hackData - contains a condition and properties/values that need to be corrected
 * @param {Object} styles - a style object
 * @param {string} property - property that gets corrected
 * @param {any} value - property value
 */

function resolveHack(hackData, styles, property, value) {

	//prefix ordinary values
	if (hackData.prefixValue) {

		var values = hackData.prefixValue[property];
		if (values) {
			if (hackData.containValue) {
				values.forEach(function (val) {
					if (value.indexOf(val) > -1) {
						styles[property] = getPrefixedValue(property, value, val);
					}
				});
			} else {
				if (values.indexOf(value) > -1) {
					styles[property] = getPrefixedValue(property, value);
				}
			}
		}
	}

	//resolve property issues
	if (hackData.alternativeProperty) {

		var oldProperty = hackData.alternativeProperty[property];
		if (oldProperty) {
			styles[oldProperty] = value;
		}
	}

	//resolve alternative values
	if (hackData.alternativeValue) {

		var oldValue = hackData.alternativeValue[property];
		if (oldValue && oldValue[value]) {
			styles[property] = oldValue[value] + ';' + camelToDashCase(property) + ':' + value;
		}
	}
}

/**
 * Generates an array of all relevant properties according to the userAgent
 * @param {string} userAgent - userAgent which gets used to gather information
 */

function generateRequiredProperties(userAgent) {
	requiredProperties = [];
	requiredHacks = [];

	if (userAgent) {
		browserinfo = (0, _browserinfo2['default'])(userAgent);
		var data = _caniuseData2['default'][browserinfo.browser.toLowerCase()];

		//only generate if there is browser data provided
		if (data) {
			var property = undefined;
			for (property in data) {
				if (data[property] >= browserinfo.version) {
					requiredProperties.push(property);
				}
			}

			//add all required hacks for current browser
			var hack = undefined;
			for (hack in _hacks2['default']) {
				var hackData = _hacks2['default'][hack](browserinfo);
				if (hackData) {
					requiredHacks.push(hackData);
				}
			}

			generated = true;
			return requiredProperties;
		} else {
			console.warn('Your browser seems to not be supported by inline-style-prefixer.');
			console.warn('Please create an issue at https://github.com/rofrischmann/inline-style-prefixer');
			return false;
		}
	} else {
		console.warn('userAgent needs to be set first. Use `.setUserAgent(userAgent)`');
		return false;
	}
}